package core.export;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.apache.log4j.Logger;

import core.structures.TodoList;

/**
 * The abstract Export class. Part of the Template Method Pattern.
 *
 *  <pre>
 *  Copyright (c) 2007 Dominik Schulz
 *
 *  This file is part of jTodoList.
 *
 *  jTodoList is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jTodoList is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with jTodoList; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *
 *  @author tex
 */
public abstract class AbstractExport implements IExport {

	static Logger		logger		= Logger.getLogger(AbstractExport.class);

	private TodoList	tdl			= null;

	private File		outputFile	= null;

	/**
	 * This method is part of the Template Method Pattern.
	 */
	public boolean export() throws IllegalStateException {
		//if(this.tdl == null || this.outputFile == null || !this.outputFile.canWrite()) {
		if (this.tdl == null || this.outputFile == null) {
			throw new IllegalStateException("Class not properly initalized.");
		}
		// export everything
		FileWriter fw = null;
		BufferedWriter bw = null;
		try {
			fw = new FileWriter(this.outputFile);
			bw = new BufferedWriter(fw);
			bw.write(this.prepareString(this.tdl));
		} catch (IOException e) {
			logger.error("Can't write to outputFile(" + this.outputFile.getAbsolutePath() + ")", e);
			return false;
		} finally {
			try {
				bw.close();
			} catch (IOException e) {
				logger.error("Can't close outputFile(" + this.outputFile.getAbsolutePath() + ")", e);
			}
			try {
				fw.close();
			} catch (IOException e) {
				logger.error("Can't close outputFile(" + this.outputFile.getAbsolutePath() + ")", e);
			}
		}
		return true;
	}

	/**
	 * This is the template method that will be implemented by the concrete subclasses.
	 * @return
	 */
	abstract String prepareString(TodoList tdl);

	public File getOutputFile() {
		return this.outputFile;
	}

	public TodoList getTodoList() {
		return this.tdl;
	}

	public boolean setOutputFile(File file) {
		if (!file.canWrite() && (!file.isFile() && !file.getParentFile().canWrite())) {
			throw new IllegalArgumentException("File not accessible");
		}
		this.outputFile = file;
		return true;
	}

	public boolean setTodoList(TodoList tdl) {
		if (tdl == null) {
			throw new IllegalArgumentException("Can set null as TodoList");
		}
		this.tdl = tdl;
		return false;
	}

}
