package core.structures;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.jdom.Content;
import org.jdom.Element;

import treemap.TMNode;
import treemap.TMUpdater;

import core.fileformat.TodoListExe;
import core.utils.XMLTools;

/**
 *  A Task.
 *
 *  <pre>
 *  Copyright (c) 2007 Dominik Schulz
 *
 *  This file is part of jTodoList.
 *
 *  jTodoList is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jTodoList is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with jTodoList; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *
 *  Implements the Composite Pattern.
 *
 *  @author tex
 */
public class Task implements ITask, TMNode {

	/**
	 * This is the base date. It's returned when some date field is null.
	 * This way an equals won't fail as it would if returning new Date() every time (which would differ).
	 */
	public static final Date	BASE_DATE	= new Date(1);

	static Logger				logger		= Logger.getLogger(Task.class);

	private Task() {
		logger.setLevel(Level.INFO);
	}

	public Task(String title) {
		logger.setLevel(Level.INFO);
		this.title = title;
	}

	/**
	 * Creates a new Task Object by passing an XML Element which describes this Task.
	 * @param xml
	 * @return
	 */
	public static ITask getTaskByXML(Element xml) {
		ITask t = new Task();
		if (xml.getAttributeValue(TodoListExe.PRIORITYCOLOR) != null)
			t.setPriorityColor(Integer.parseInt(xml.getAttributeValue(TodoListExe.PRIORITYCOLOR)));
		//		t.setStartDate(new Date(Date.parse(xml.getAttributeValue(STARTDATESTRING)))); // TODO Date konvertieren
		if (xml.getAttributeValue(TodoListExe.RISK) != null)
			t.setRisk(Integer.parseInt(xml.getAttributeValue(TodoListExe.RISK)));
		//t.setLastModifiedDate(new Date(Date.parse(xml.getAttributeValue(LASTMODSTRING)))); // TODO Date konvertieren
		if (xml.getAttributeValue(TodoListExe.PRIORITY) != null)
			t.setPriority(Integer.parseInt(xml.getAttributeValue(TodoListExe.PRIORITY)));
		t.setStatus(xml.getAttributeValue(TodoListExe.STATUS));
		//t.setDueDate(new Date(Date.parse(xml.getAttributeValue(DUEDATESTRING))));  // TODO Date konvertieren
		t.setAllocatedBy(xml.getAttributeValue(TodoListExe.ALLOCATEDBY));
		//t.setLastModifiedDate(new Date(Date.parse(xml.getAttributeValue(LASTMOD))));  // TODO Date konvertieren
		//t.setDoneDate(new Date(Date.parse(xml.getAttributeValue(CREATIONDATESTRING)))); // TODO Date konvertieren
		t.setTitle(xml.getAttributeValue(TodoListExe.TITLE));
		t.setTimeSpentUnits(xml.getAttributeValue(TodoListExe.TIMESPENTUNITS));
		t.setPerson(xml.getAttributeValue(TodoListExe.PERSON));
		if (xml.getAttributeValue(TodoListExe.TIMEESTIMATE) != null)
			t.setTimeEstimated(Double.parseDouble(xml.getAttributeValue(TodoListExe.TIMEESTIMATE)));
		if (xml.getAttributeValue(TodoListExe.NUMCATEGORY) != null)
			t.setNumCategory(Integer.parseInt(xml.getAttributeValue(TodoListExe.NUMCATEGORY)));
		t.setCategory(xml.getAttributeValue(TodoListExe.CATEGORY));
		t.setComments(xml.getAttributeValue(TodoListExe.COMMENTS));
		t.setPriorityWebColor(xml.getAttributeValue(TodoListExe.PRIORITYWEBCOLOR));
		if (xml.getAttributeValue(TodoListExe.TIMESPENT) != null)
			t.setTimeSpent(Double.parseDouble(xml.getAttributeValue(TodoListExe.TIMESPENT)));
		t.setFileReferingPath(xml.getAttributeValue(TodoListExe.FILEREFPATH));
		if (xml.getAttributeValue(TodoListExe.ID) != null)
			t.setId(Integer.parseInt(xml.getAttributeValue(TodoListExe.ID)));
		//t.setDueDate(new Date(Date.parse(xml.getAttributeValue(DUEDATE)))); // TODO Date konvertieren
		if (xml.getAttributeValue(TodoListExe.COST) != null)
			t.setCost(Double.parseDouble(xml.getAttributeValue(TodoListExe.COST)));
		if (xml.getAttributeValue(TodoListExe.PERCENTDONE) != null)
			t.setPercentdone(Integer.parseInt(xml.getAttributeValue(TodoListExe.PERCENTDONE)));
		t.setDepends(xml.getAttributeValue(TodoListExe.DEPENDS));
		//t.setStartDate(new Date(Date.parse(xml.getAttributeValue(STARTDATESTRING)))); // TODO Date konvertieren
		//t.setDoneDate(new Date(Date.parse(xml.getAttributeValue(DONEDATESTRING)))); // TODO Date konvertieren
		if (xml.getAttributeValue(TodoListExe.POS) != null)
			t.setPos(Integer.parseInt(xml.getAttributeValue(TodoListExe.POS)));
		for (Iterator iter = xml.getChildren().iterator(); iter.hasNext();) {
			Element element = (Element) iter.next();
			if (element == null)
				continue;
			t.add(Task.getTaskByXML(element));
		}
		return t;
	}

	private int		priorityColor		= 0;

	private Date	startDate			= new Date();	// used for startdate and startdatestring

	private int		risk				= 0;

	private Date	lastModifiedDate	= new Date();	// used for lastmod and lastmodstring

	private int		priority			= 5;			// 0-10, default: 5

	/**
	 * A custom status message, e.g. GOOD, BAD, NICE, GREAT ...
	 */
	private String	status				= "";

	/**
	 * A due date, e.g. deadline
	 */
	private Date	dueDate				= new Date();	// used for duedate and duedatestring

	/**
	 * Whom is this task allocated
	 */
	private String	allocatedBy			= "";

	/**
	 * When was this task created
	 */
	private Date	creationDate		= new Date();	// used for creationdate and creationdatestring

	/**
	 * The Title of this task. Probably the most important field.
	 */
	private String	title				= "";

	/**
	 * The unit of the field timeSpent, e.g. H (=HOURS), M (=MINUTES), D (=DAYS), ...
	 */
	private String	timeSpentUnits		= "H";

	/**
	 * An external ID, e.g. a Request-Tracker-ID
	 */
	private String	externalId			= "";

	/**
	 * The unit filed of timeEstimated. Same units as timeSpent.
	 */
	private String	timeEstimatedUnits	= "H";

	/**
	 * This person who maintains this task.
	 */
	private String	person				= "";

	/**
	 * The estimated time for this task in the units of timeEstimatedUnits.
	 */
	private double	timeEstimated		= 0.0;

	private int		numCategory			= 0;

	/**
	 * A category for this task.
	 */
	private String	category			= "";

	/**
	 * A Field containing comments and/or descriptive text.
	 */
	private String	comments			= "";

	private String	priorityWebColor	= "";

	/**
	 * This time spent working on this task in the unit of timeSpentUnit.
	 */
	private double	timeSpent			= 0.0;

	/**
	 * The Path to a associated file.
	 */
	private String	fileReferingPath	= "";

	/**
	 * The internal ID of this task.
	 */
	private int		id					= 0;

	/**
	 * The actual or estimated cost of this task.
	 */
	private double	cost				= 0.0;

	/**
	 * The completition-status of this task.
	 */
	private int		percentdone			= 0;

	/**
	 * What does this task depend on?
	 */
	private String	depends				= "";

	/**
	 * The position of this task relative to the other tasks on the same level.
	 */
	private int		pos					= 0;

	/**
	 * When was this task completed?
	 */
	private Date	doneDate			= new Date();	// used for donedate and donedatestring

	/* (non-Javadoc)
	 * @see core.structures.ITask#equals(java.lang.Object)
	 */
	public boolean equals(Object o) {
		ITask other = (ITask) o;
		logger.trace("equals() - comparing this (" + this + ") to that (" + other + ")");
		try {
			if (other.getPriorityColor() != this.getPriorityColor()) {
				logger.trace("Mismatch - prio color");
				return false;
			}
			if (!other.getStartDate().equals(this.getStartDate())) {
				logger.trace("Mismatch - start date");
				return false;
			}
			if (other.getRisk() != this.getRisk()) {
				logger.trace("Mismatch - risk");
				return false;
			}
			if (!other.getLastModifiedDate().equals(this.getLastModifiedDate())) {
				logger.trace("Mismatch - last mod date");
				return false;
			}
			if (other.getPriority() != (this.getPriority())) {
				logger.trace("Mismatch - prio");
				return false;
			}
			if (!other.getStatus().equals(this.getStatus())) {
				logger.trace("Mismatch - status");
				return false;
			}
			if (!other.getDueDate().equals(this.getDueDate())) {
				logger.trace("Mismatch - due date");
				return false;
			}
			if (!other.getAllocatedBy().equals(this.getAllocatedBy())) {
				logger.trace("Mismatch - alloc by");
				return false;
			}
			if (!other.getCreationDate().equals(this.getCreationDate())) {
				logger.trace("Mismatch - creation date");
				return false;
			}
			if (!other.getTitle().equals(this.getTitle())) {
				logger.trace("Mismatch - title");
				return false;
			}
			if (!other.getTimeSpentUnits().equals(this.getTimeSpentUnits())) {
				logger.trace("Mismatch - time spent units");
				return false;
			}
			if (!other.getExternalId().equals(this.getExternalId())) {
				logger.trace("Mismatch -ext. id");
				return false;
			}
			if (!other.getTimeEstimatedUnits().equals(this.getTimeEstimatedUnits())) {
				logger.trace("Mismatch- time est units");
				return false;
			}
			if (!other.getPerson().equals(this.getPerson())) {
				logger.trace("Mismatch - Person");
				return false;
			}

			if (other.getTimeEstimated() != this.getTimeEstimated()) {
				logger.trace("Mismatch - timeEstimated");
				return false;
			}
			if (other.getNumCategory() != this.getNumCategory()) {
				logger.trace("Mismatch - NumCategory");
				return false;
			}
			if (!other.getCategory().equals(this.getCategory())) {
				logger.trace("Mismatch - Category");
				return false;
			}
			if (!other.getComments().equals(this.getComments())) {
				logger.trace("Mismatch - Comments");
				return false;
			}
			if (!other.getPriorityWebColor().equals(this.getPriorityWebColor())) {
				logger.trace("Mismatch - prio web color");
				return false;
			}
			if (other.getTimeSpent() != this.getTimeSpent()) {
				logger.trace("Mismatch - time spent");
				return false;
			}
			if (!other.getFileReferingPath().equals(this.getFileReferingPath())) {
				logger.trace("Mismatch - file ref path");
				return false;
			}
			if (other.getId() != this.getId()) {
				logger.trace("Mismatch - id");
				return false;
			}
			if (other.getCost() != this.getCost()) {
				logger.trace("Mismatch - cost");
				return false;
			}
			if (other.getPercentdone() != this.getPercentdone()) {
				logger.trace("Mismatch - percent done");
				return false;
			}
			if (!other.getDepends().equals(this.getDepends())) {
				logger.trace("Mismatch - depends");
				return false;
			}
			if (other.getPos() != this.getPos()) {
				logger.trace("Mismatch - pos");
				return false;
			}
			if (!other.getDoneDate().equals(this.getDoneDate())) {
				logger.trace("Mismatch - done date");
				return false;
			}
			for (int i = 0; i < this.tasks.size(); i++) {
				if (!this.tasks.get(i).equals(other.getTask(i))) {
					logger.trace("Mismatch - this (" + this.tasks.get(i) + ") to that(" + other.getTask(i) + ")");
					return false;
				}
			}
			return true;
		} catch (NullPointerException e) {
			logger.error("Got NullPointerException", e);
			return false;
		}
	}

	/**
	 * The Subtasks of the Task-Node
	 */
	private ArrayList<ITask>	tasks	= new ArrayList<ITask>();

	/* (non-Javadoc)
	 * @see core.structures.ITask#add(int, java.lang.Object)
	 */
	public void add(int index, Object element) {
		if (element instanceof ITask)
			this.tasks.add(index, (ITask) element);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#add(java.lang.Object)
	 */
	public boolean add(Object o) {
		if (o instanceof ITask)
			return this.tasks.add((ITask) o);
		else
			return false;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#addAll(java.util.Collection)
	 */
	@SuppressWarnings("unchecked")
	public boolean addAll(Collection c) {
		return this.tasks.addAll(c);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#addAll(int, java.util.Collection)
	 */
	@SuppressWarnings("unchecked")
	public boolean addAll(int index, Collection c) {
		return this.tasks.addAll(index, c);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#addTask(core.structures.Task)
	 */
	public void addTask(ITask t) {
		logger.trace("Adding Task " + t + " to " + this);
		this.tasks.add(t);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#clear()
	 */
	public void clear() {
		this.tasks.clear();
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#contains(java.lang.Object)
	 */
	public boolean contains(Object o) {
		return this.tasks.contains(o);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#containsAll(java.util.Collection)
	 */
	public boolean containsAll(Collection c) {
		return this.tasks.containsAll(c);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#get(int)
	 */
	public Object get(int index) {
		return this.tasks.get(index);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getAllocatedBy()
	 */
	public String getAllocatedBy() {
		return (allocatedBy == null) ? new String() : this.allocatedBy;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getAsTreeNode()
	 */
	public DefaultMutableTreeNode getAsTreeNode() {
		DefaultMutableTreeNode node = new DefaultMutableTreeNode(this.getTitle());
		node.setUserObject(this);
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			ITask element = (ITask) iter.next();
			node.add(element.getAsTreeNode());
		}
		return node;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getCategory()
	 */
	public String getCategory() {
		return (category == null) ? new String() : this.category;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getComments()
	 */
	public String getComments() {
		return (comments == null) ? new String() : this.comments;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getCost()
	 */
	public double getCost() {
		return cost;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getCreationDate()
	 */
	public Date getCreationDate() {
		return (creationDate == null) ? BASE_DATE : this.creationDate;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getDepends()
	 */
	public String getDepends() {
		return (depends == null) ? new String() : this.depends;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getDoneDate()
	 */
	public Date getDoneDate() {
		return (doneDate == null) ? BASE_DATE : this.doneDate;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getDueDate()
	 * FIXME Eigentlich sollte immer ein festes Datum zurückgegeben werden damit ein Vergleich nicht scheitert.
	 */
	public Date getDueDate() {
		return (dueDate == null) ? BASE_DATE : this.dueDate;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getExternalId()
	 */
	public String getExternalId() {
		return (externalId == null) ? new String() : this.externalId;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getFileReferingPath()
	 */
	public String getFileReferingPath() {
		return (fileReferingPath == null) ? new String() : this.fileReferingPath;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getId()
	 */
	public int getId() {
		return id;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getLastModifiedDate()
	 */
	public Date getLastModifiedDate() {
		return (lastModifiedDate == null) ? BASE_DATE : this.lastModifiedDate;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getNumCategory()
	 */
	public int getNumCategory() {
		return numCategory;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getPercentdone()
	 */
	public int getPercentdone() {
		return percentdone;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getPerson()
	 */
	public String getPerson() {
		return (person == null) ? new String() : this.person;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getPos()
	 */
	public int getPos() {
		return pos;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getPriority()
	 */
	public int getPriority() {
		return priority;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getPriorityColor()
	 */
	public int getPriorityColor() {
		return priorityColor;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getPriorityWebColor()
	 */
	public String getPriorityWebColor() {
		return (priorityWebColor == null) ? new String() : this.priorityWebColor;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getRisk()
	 */
	public int getRisk() {
		return risk;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getStartDate()
	 */
	public Date getStartDate() {
		return (startDate == null) ? BASE_DATE : this.startDate;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getStatus()
	 */
	public String getStatus() {
		return (status == null) ? new String() : this.status;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getTask(int)
	 */
	public ITask getTask(int i) {
		return this.tasks.get(i);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getTimeEstimated()
	 */
	public double getTimeEstimated() {
		return timeEstimated;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getTimeEstimatedUnits()
	 */
	public String getTimeEstimatedUnits() {
		return (timeEstimatedUnits == null) ? new String() : this.timeEstimatedUnits;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getTimeSpent()
	 */
	public double getTimeSpent() {
		return timeSpent;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getTimeSpentUnits()
	 */
	public String getTimeSpentUnits() {
		return (timeSpentUnits == null) ? new String() : this.timeSpentUnits;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getTitle()
	 */
	public String getTitle() {
		return (title == null) ? new String() : this.title;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#getXML()
	 */
	public Content getXML() {
		Element node = new Element(TodoListExe.TASK);
		node.setAttribute(TodoListExe.PRIORITYCOLOR, new Integer(this.priorityColor).toString());
		if (this.startDate != null)
			node.setAttribute(TodoListExe.STARTDATESTRING, this.startDate.toString()); // TODO Date konvertieren
		node.setAttribute(TodoListExe.RISK, new Integer(this.risk).toString());
		if (this.doneDate != null)
			node.setAttribute(TodoListExe.LASTMODSTRING, this.doneDate.toString()); // TODO Date konvertieren
		node.setAttribute(TodoListExe.PRIORITY, new Integer(this.priority).toString());
		if (this.status != null)
			node.setAttribute(TodoListExe.STATUS, this.status);
		if (this.dueDate != null)
			node.setAttribute(TodoListExe.DUEDATESTRING, this.dueDate.toString());
		// TODO Calcpercentdone
		if (this.allocatedBy != null)
			node.setAttribute(TodoListExe.ALLOCATEDBY, this.allocatedBy);
		if (this.lastModifiedDate != null)
			node.setAttribute(TodoListExe.LASTMOD, this.lastModifiedDate.toString()); // TODO Date konvertieren
		if (this.creationDate != null)
			node.setAttribute(TodoListExe.CREATIONDATESTRING, this.creationDate.toString()); // TODO Date konvertieren
		if (this.title != null)
			node.setAttribute(TodoListExe.TITLE, this.title);
		if (this.creationDate != null)
			node.setAttribute(TodoListExe.CREATIONDATE, this.creationDate.toString()); // TODO Date konvertieren
		if (this.timeSpentUnits != null)
			node.setAttribute(TodoListExe.TIMESPENTUNITS, this.timeSpentUnits);
		if (this.person != null)
			node.setAttribute(TodoListExe.PERSON, this.person);
		node.setAttribute(TodoListExe.TIMEESTIMATE, new Double(this.timeEstimated).toString());
		// TODO Calctimespent
		// TODO numcategory
		if (this.category != null)
			node.setAttribute(TodoListExe.CATEGORY, this.category);
		if (this.comments != null)
			node.setAttribute(TodoListExe.COMMENTS, this.comments);
		node.setAttribute(TodoListExe.PRIORITYWEBCOLOR, this.priorityWebColor);
		node.setAttribute(TodoListExe.TIMESPENT, new Double(this.timeSpent).toString());
		if (this.fileReferingPath != null)
			node.setAttribute(TodoListExe.FILEREFPATH, this.fileReferingPath);
		node.setAttribute(TodoListExe.ID, new Integer(this.id).toString());
		if (this.dueDate != null)
			node.setAttribute(TodoListExe.DUEDATE, this.dueDate.toString());
		node.setAttribute(TodoListExe.COST, new Double(this.cost).toString());
		// TODO customcomments
		// TODO calctimestimate
		// TODO calccost
		node.setAttribute(TodoListExe.PERCENTDONE, new Integer(this.percentdone).toString());
		if (this.depends != null)
			node.setAttribute(TodoListExe.DEPENDS, this.depends);
		if (this.startDate != null)
			node.setAttribute(TodoListExe.STARTDATE, this.startDate.toString()); // TODO Date konvertieren
		if (this.doneDate != null)
			node.setAttribute(TodoListExe.DONEDATE, this.doneDate.toString()); // TODO Date konvertieren
		if (this.doneDate != null)
			node.setAttribute(TodoListExe.DONEDATESTRING, this.doneDate.toString()); // TODO Date konvertieren
		node.setAttribute(TodoListExe.POS, new Integer(this.pos).toString());
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			ITask element = (ITask) iter.next();
			node.addContent(element.getXML());
		}
		return node;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#hashCode()
	 */
	public int hashCode() {
		int hc = 0;
		hc += this.id;
		hc += this.numCategory;
		hc += this.percentdone;
		hc += this.pos;
		hc += this.priority;
		hc += this.priorityColor;
		hc += this.risk;
		hc += this.allocatedBy.hashCode();
		hc += this.category.hashCode();
		hc += this.comments.hashCode();
		hc += this.cost;
		hc += this.creationDate.hashCode();
		hc += this.depends.hashCode();
		hc += this.doneDate.hashCode();
		hc += this.dueDate.hashCode();
		hc += this.externalId.hashCode();
		hc += this.fileReferingPath.hashCode();
		hc += this.lastModifiedDate.hashCode();
		hc += this.numCategory;
		hc += this.percentdone;
		hc += this.pos;
		hc += this.priority;
		hc += this.priorityColor;
		hc += this.person.hashCode();
		hc += this.risk;
		hc += this.startDate.hashCode();
		hc += this.status.hashCode();
		hc += this.timeEstimated;
		hc += this.timeEstimatedUnits.hashCode();
		hc += this.timeSpent;
		hc += this.timeSpentUnits.hashCode();
		hc += this.title.hashCode();
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			Task element = (Task) iter.next();
			hc += element.hashCode();
		}
		return hc;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#indexOf(java.lang.Object)
	 */
	public int indexOf(Object o) {
		return this.tasks.indexOf(o);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#isEmpty()
	 */
	public boolean isEmpty() {
		return this.tasks.isEmpty();
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#iterator()
	 */
	public Iterator iterator() {
		return this.tasks.iterator();
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#lastIndexOf(java.lang.Object)
	 */
	public int lastIndexOf(Object o) {
		return this.tasks.lastIndexOf(o);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#listIterator()
	 */
	public ListIterator listIterator() {
		return this.tasks.listIterator();
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#listIterator(int)
	 */
	public ListIterator listIterator(int index) {
		return this.listIterator(index);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#numTasks()
	 */
	public int numTasks() {
		return this.tasks.size();
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#prettyFormat(int)
	 */
	public String prettyFormat(int indent) {
		String str = "";
		str += "Task: " + this.title + " Priority: " + this.priority + '\n';
		boolean wasInLoop = false;
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			ITask element = (ITask) iter.next();
			str += XMLTools.getNumChars('-', indent) + element.prettyFormat(indent + 1) + '\n';
			wasInLoop = true;
		}
		if (wasInLoop)
			str = str.substring(0, str.length() - 1); // remove last \n
		return str;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#remove(int)
	 */
	public Object remove(int index) {
		return this.tasks.remove(index);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#remove(java.lang.Object)
	 */
	public boolean remove(Object o) {
		if (o instanceof ITask) {
			return this.removeTask((ITask) o);
		} else {
			return this.tasks.remove(o);
		}
	}

	/**
	 * This will remove an Task
	 */
	public boolean removeTask(ITask t) {
		if (this.tasks.contains(t)) {
			return this.tasks.remove(t);
		} else {
			for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
				ITask element = (ITask) iter.next();
				if (element.removeTask(t)) {
					return true;
				}
			}
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#removeAll(java.util.Collection)
	 */
	public boolean removeAll(Collection c) {
		return this.tasks.removeAll(c);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#retainAll(java.util.Collection)
	 */
	public boolean retainAll(Collection c) {
		return this.tasks.retainAll(c);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#set(int, java.lang.Object)
	 */
	public Object set(int index, Object element) {
		if (element instanceof Task)
			return this.tasks.set(index, (Task) element);
		else
			return null;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setAllocatedBy(java.lang.String)
	 */
	public void setAllocatedBy(String allocatedBy) {
		this.allocatedBy = allocatedBy;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setCategory(java.lang.String)
	 */
	public void setCategory(String category) {
		this.category = category;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setComments(java.lang.String)
	 */
	public void setComments(String comments) {
		this.comments = comments;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setCost(double)
	 */
	public void setCost(double cost) {
		this.cost = cost;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setCreationDate(java.util.Date)
	 */
	public void setCreationDate(Date creationDate) {
		this.creationDate = creationDate;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setDepends(java.lang.String)
	 */
	public void setDepends(String depends) {
		this.depends = depends;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setDoneDate(java.util.Date)
	 */
	public void setDoneDate(Date doneDate) {
		this.doneDate = doneDate;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setDueDate(java.util.Date)
	 */
	public void setDueDate(Date dueDate) {
		this.dueDate = dueDate;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setExternalId(java.lang.String)
	 */
	public void setExternalId(String externalId) {
		this.externalId = externalId;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setFileReferingPath(java.lang.String)
	 */
	public void setFileReferingPath(String fileReferingPath) {
		this.fileReferingPath = fileReferingPath;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setId(int)
	 */
	public void setId(int id) {
		this.id = id;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setLastModifiedDate(java.util.Date)
	 */
	public void setLastModifiedDate(Date lastModifiedDate) {
		this.lastModifiedDate = lastModifiedDate;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setNumCategory(int)
	 */
	public void setNumCategory(int numCategory) {
		this.numCategory = numCategory;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setPercentdone(int)
	 */
	public void setPercentdone(int percentdone) {
		this.percentdone = percentdone;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setPerson(java.lang.String)
	 */
	public void setPerson(String person) {
		this.person = person;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setPos(int)
	 */
	public void setPos(int pos) {
		this.pos = pos;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setPriority(int)
	 */
	public void setPriority(int priority) {
		this.priority = priority;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setPriorityColor(int)
	 */
	public void setPriorityColor(int priorityColor) {
		this.priorityColor = priorityColor;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setPriorityWebColor(java.lang.String)
	 */
	public void setPriorityWebColor(String priorityWebColor) {
		this.priorityWebColor = priorityWebColor;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setRisk(int)
	 */
	public void setRisk(int risk) {
		this.risk = risk;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setStartDate(java.util.Date)
	 */
	public void setStartDate(Date startDate) {
		this.startDate = startDate; // TODO implement
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setStatus(java.lang.String)
	 */
	public void setStatus(String status) {
		this.status = status;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setTimeEstimated(double)
	 */
	public void setTimeEstimated(double timeEstimated) {
		this.timeEstimated = timeEstimated;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setTimeEstimatedUnits(java.lang.String)
	 */
	public void setTimeEstimatedUnits(String timeEstimatedUnits) {
		this.timeEstimatedUnits = timeEstimatedUnits;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setTimeSpent(double)
	 */
	public void setTimeSpent(double timeSpent) {
		this.timeSpent = timeSpent;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setTimeSpentUnits(java.lang.String)
	 */
	public void setTimeSpentUnits(String timeSpentUnits) {
		this.timeSpentUnits = timeSpentUnits;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#setTitle(java.lang.String)
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#size()
	 */
	public int size() {
		return this.tasks.size();
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#subList(int, int)
	 */
	public List subList(int fromIndex, int toIndex) {
		return this.tasks.subList(fromIndex, toIndex);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#toArray()
	 */
	public Object[] toArray() {
		return this.tasks.toArray();
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#toArray(java.lang.Object[])
	 */
	@SuppressWarnings("unchecked")
	public Object[] toArray(Object[] a) {
		return this.tasks.toArray(a);
	}

	/* (non-Javadoc)
	 * @see core.structures.ITask#toString()
	 */
	public String toString() {
		String str = "";
		str = this.title;
		return str;
	}

	public Collection<? extends ITask> getAsList() {
		/**
		 * @return a list of tasks
		 */
		List<ITask> tasks = new Vector<ITask>();
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			ITask element = (ITask) iter.next();
			if (element.numTasks() > 0) {
				tasks.addAll(element.getAsList());
			}
			tasks.add(element);
		}
		return tasks;
	}

	public Enumeration children() {
		Vector<ITask> children = new Vector<ITask>();
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			ITask element = (ITask) iter.next();
			children.add(element);
			logger.trace("children(): Added this: " + element.getTitle());
		}
		return children.elements();
	}

	public boolean isLeaf() {
		logger.trace("isLeaf() - has this many children: " + this.tasks.size());
		if(this.tasks.size() > 0)
			return false; // has children
		else
			return true; // has no children
	}

	public void setUpdater(TMUpdater arg0) {
		// TODO Auto-generated method stub

	}

}
