package core.structures;

import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;

import treemap.TMNode;
import treemap.TMUpdater;

/**
 * A ToDoList
 *  <pre>
 *  Copyright (c) 2007 Dominik Schulz
 *
 *  This file is part of jTodoList.
 *
 *  jTodoList is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  jTodoList is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with jTodoList; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  </pre>
 *
 *  @author tex
 */
public class TodoList implements TMNode {

	static Logger				logger				= Logger.getLogger(TodoList.class);

	private ArrayList<ITask>	tasks				= new ArrayList<ITask>();

	/**
	 * The Fileformat version
	 */
	private int					fileformat			= 8;

	/**
	 * The Project Name
	 */
	private String				projectName			= "";

	/**
	 * The next unique id
	 */
	private int					nextuniqeid			= 0;

	/**
	 * The fileversion
	 */
	private int					fileversion			= 0;

	/**
	 * When was this list the last time modified?
	 */
	private Date				lastModified		= null;

	/**
	 * The field custom comments type
	 */
	private String				customCommentsType	= "";

	private TodoList() {
		logger.setLevel(Level.INFO); // only for internal use
	}

	/**
	 * For extenal use.
	 * @param projectName
	 */
	public TodoList(String projectName) {
		logger.setLevel(Level.INFO);
		this.projectName = projectName;
	}

	/**
	 * @see Object.hashCode()
	 */
	public int hashCode() {
		int hc = 0;
		hc += this.fileformat;
		hc += this.projectName.hashCode();
		hc += this.nextuniqeid;
		hc += this.fileversion;
		hc += this.lastModified.hashCode();
		hc += this.customCommentsType.hashCode();
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			Task element = (Task) iter.next();
			hc += element.hashCode();
		}
		return hc;
	}

	/**
	 * @see Object.equals(Object)
	 */
	public boolean equals(Object o) {
		TodoList other = (TodoList) o;
		if (other.getFileformat() != this.getFileformat()) {
			logger.trace("equals - Fileformats differ: this: |" + this.getFileformat() + "|, other: |"
					+ other.getFileformat() + "|");
			return false;
		}
		if (!other.getProjectName().equals(this.getProjectName())) {
			logger.trace("equals - ProjectName differ: this " + this.getProjectName() + ", other: "
					+ other.getProjectName());
			return false;
		}
		if (other.getNextuniqeid() != this.getNextuniqeid()) {
			logger.trace("equals - NexUniqueId differ: this " + this.getNextuniqeid() + ", other: "
					+ other.getNextuniqeid());
			return false;
		}
		if (other.getFileversion() != this.getFileversion()) {
			logger.trace("equals - Fileversion differ: this " + this.getFileversion() + ", other: "
					+ other.getFileversion());
			return false;
		}
		if (!other.getLastModified().equals(this.getLastModified())) {
			logger.trace("equals - Last Mod differ: this " + this.getLastModified() + ", other: "
					+ other.getLastModified());
			return false;
		}
		for (int i = 0; i < this.tasks.size(); i++) {
			if (!this.tasks.get(i).equals(other.getTask(i)))
				return false;
		}
		logger.trace("equals - everything ok");
		return true;
	}

	/**
	 * @return the customCommentsType
	 */
	public String getCustomCommentsType() {
		return customCommentsType;
	}

	/**
	 * @param customCommentsType the customCommentsType to set
	 */
	public void setCustomCommentsType(String customCommentsType) {
		this.customCommentsType = customCommentsType;
	}

	/**
	 * @return the fileformat
	 */
	public int getFileformat() {
		return fileformat;
	}

	/**
	 * @param fileformat the fileformat to set
	 */
	public void setFileformat(int fileformat) {
		this.fileformat = fileformat;
	}

	/**
	 * @return the fileversion
	 */
	public int getFileversion() {
		return fileversion;
	}

	/**
	 * @param fileversion the fileversion to set
	 */
	public void setFileversion(int fileversion) {
		this.fileversion = fileversion;
	}

	/**
	 * @return the lastmodified
	 */
	public Date getLastModified() {
		return (lastModified == null) ? new Date() : this.lastModified;
	}

	/**
	 * @param lastmodified the lastmodified to set
	 */
	public void setLastmodified(Date lastmodified) {
		this.lastModified = lastmodified;
	}

	/**
	 * @return the nextuniqeid
	 */
	public int getNextuniqeid() {
		return nextuniqeid;
	}

	/**
	 * @param nextuniqeid the nextuniqeid to set
	 */
	public void setNextuniqeid(int nextuniqeid) {
		this.nextuniqeid = nextuniqeid;
	}

	public String toString() {
		String str = "";
		str = this.projectName;
		str = " " + this.fileformat;
		str = " " + this.fileversion;
		return str;
	}

	public JTree getAsTree() {
		DefaultMutableTreeNode root = new DefaultMutableTreeNode();
		root.setUserObject(this);
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			ITask element = (ITask) iter.next();
			root.add(element.getAsTreeNode());
		}
		JTree jt = new JTree(root);
		return jt;
	}

	/**
	 * @return a list of tasks
	 */
	public List<ITask> getAsList() {
		List<ITask> tasks = new Vector<ITask>();
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			ITask element = (ITask) iter.next();
			if (element.numTasks() > 0) {
				tasks.addAll(element.getAsList());
			}
			tasks.add(element);
		}
		return tasks;
	}

	public ITask[] getAsArray() {
		ITask[] a = new ITask[1];
		a = this.getAsList().toArray(a);
		return a;
	}

	public void setXML(Document xml) {
		Element root = xml.getRootElement();
		logger.trace(root.getName());
		if (root.getAttributeValue("FILEFORMAT") != null)
			this.fileformat = Integer.parseInt(root.getAttributeValue("FILEFORMAT"));
		if (root.getAttributeValue("FILEVERSION") != null)
			this.fileversion = Integer.parseInt(root.getAttributeValue("FILEVERSION"));
		if (root.getAttributeValue("NEXTUNIQUEID") != null)
			this.nextuniqeid = Integer.parseInt(root.getAttributeValue("NEXTUNIQUEID"));
		this.projectName = root.getAttributeValue("PROJECTNAME");
		//this.lastmodified = root.getAttributeValue("LASTMODIFIED"); // TODO Parse DATE
		this.customCommentsType = root.getAttributeValue("CUSTOMCOMMENTSTYPE");
		for (Iterator iter = root.getChildren().iterator(); iter.hasNext();) {
			Element element = (Element) iter.next();
			if (element == null)
				continue;
			this.tasks.add(Task.getTaskByXML(element));
		}
	}

	public Element getXML() {
		Element root = new Element("TODOLIST");
		root.setAttribute("FIELFORMAT", new Integer(this.fileformat).toString());
		root.setAttribute("FILEVERSION", new Integer(this.fileversion).toString());
		root.setAttribute("NEXTUNIQUEID", new Integer(this.nextuniqeid).toString());
		if (this.lastModified != null)
			root.setAttribute("LASTMODIFIED", this.lastModified.toString());
		else
			root.setAttribute("LASTMODIFIED", "");
		root.setAttribute("CUSTOMCOMMENTSTYPE", this.customCommentsType);
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			ITask element = (ITask) iter.next();
			root.addContent(element.getXML());
		}
		return root;
	}

	public void addTask(ITask t) {
		this.tasks.add(t);
	}

	public ITask getTask(int i) {
		return this.tasks.get(i);
	}

	public int numTasks() {
		return this.tasks.size();
	}

	/**
	 * @return the projectName
	 */
	public String getProjectName() {
		return projectName;
	}

	/**
	 * @param projectName the projectName to set
	 */
	public void setProjectName(String projectName) {
		this.projectName = projectName;
	}

	/**
	 * @param lastModified the lastModified to set
	 */
	public void setLastModified(Date lastModified) {
		this.lastModified = lastModified;
	}

	/**
	 * @see List.remove(Object)
	 * @param o
	 * @return
	 */
	public boolean remove(Object o) {
		logger.trace("Removing Task " + o);
		if (o instanceof ITask) {
			ITask t = (ITask) o;
			for (int i = 0; i < this.tasks.size(); i++) {
				if (this.tasks.get(i).equals(t)) {
					this.tasks.remove(i);
					logger.trace("Removed Task!");
					return true;
				} else {
					if (this.tasks.get(i).removeTask(t)) {
						logger.trace("Removed Task!");
						return true;
					}
				}
			}
			logger.trace("Removing failed! (not found)");
			return false;
		} else {
			logger.trace("Removing failed! (wrong instance type)");
			return false;
		}
	}

	public Enumeration children() {
		Vector<ITask> children = new Vector<ITask>();
		for (Iterator iter = this.tasks.iterator(); iter.hasNext();) {
			ITask element = (ITask) iter.next();
			children.add(element);
			logger.trace("children(): Added this: " + element.getTitle());
		}
		return children.elements();
	}

	public boolean isLeaf() {
		logger.trace("isLeaf() - has this many children: " + this.tasks.size());
		if(this.tasks.size() > 0)
			return false; // has children
		else
			return true; // has no children
	}

	public void setUpdater(TMUpdater arg0) {
		// TODO Auto-generated method stub
	}

}
