package core.utils;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * Common XML Stuff.
 *
 * <pre>
 *          Copyright (c) 2006 Sebastian Erdweg
 *          Copyright (c) 2006-2007 Dominik Schulz
 *          Copyright (c) 2006 Florian Lindner
 *          Copyright (c) 2006 Betim Berjani
 *
 *          This file is part of FlexiCrypt.
 *
 *          FlexiCrypt is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          FlexiCrypt is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with FlexiCrypt; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author Dominik
 *
 * This class contains conveinence methods for handling XML. It was designed during the Bachelor-Lab and adopted for
 * several other projects.
 *
 *
 * "Chuck Norris is so fast, he can run around the world and punch himself in the back of the head."
 */
public class XMLTools {

	public final static String	FC_TEMPFILE_EXT	= "fctmp";

	/**
	 * Descend into the XML-Tree and find a node with the given name
	 *
	 * @param nodes
	 * @param name
	 * @return null if nothing found, else the element
	 */
	public static Element getXMLNode(List nodes, String name) {
		for (int i = 0; i < nodes.size(); i++) {
			Element element = (Element) nodes.get(i);
			if (element == null) {
				continue;
			}
			if (!element.getName().equals(name)) {
				// descend further
				Element returned = getXMLNode(element.getChildren(), name);
				if (returned != null) {
					// if found sth, just break
					return returned;
				}
				continue;
			}
			return element;
		}
		return null;
	}

	/**
	 * Parse an URI-String a return a local File object this method parses an URI/URL an returns a File-Object e.g. if
	 * the URI references a local object the path is resolved and converted to an File object or if the URI references a
	 * remote object, the object is fetched and an file object is created on the fetched file
	 *
	 * @link http://java.sun.com/j2se/1.5.0/docs/api/java/net/URI.html
	 * @link http://java.sun.com/j2se/1.5.0/docs/api/java/net/URL.html
	 * @param uriPath
	 * @return the File referenced by the URI
	 * @throws URISyntaxException
	 * @throws IOException
	 */
	public static File getFileFromURI(String uriPath) throws URISyntaxException, IOException {
		File file = null;

		// 1.) get url from uri
		URI uri = new URI(uriPath);
		URL url = uri.toURL();
		// 2.) local file or remote object? local -> create file object, remote -> download to temp dir and create file
		// object
		if (url.getProtocol().equals("file")) {
			// 2.1) local file
			file = new File(url.getPath());
		} else {
			// 2.2) remote file
			DataInputStream fis = new DataInputStream(url.openStream());
			String tempFileName = getTempFilename();
			file = new File(tempFileName);
			file.deleteOnExit(); // delete the temp file on exit (of the vm)
			FileOutputStream fos = new FileOutputStream(file);
			byte[] buff = new byte[16];
			int i = 0;
			while ((i = fis.read(buff)) > -1) {
				fos.write(buff, 0, i);
			}
			fos.close();
			fis.close();
		}
		return file;
	}

	/**
	 * This mehtod generates a unique filename for temp files.
	 *
	 * @return a unique filename for temp files
	 */
	public static String getTempFilename() {
		int count = 0;
		String tempFilename = getTempFilename(count);
		File f = new File(tempFilename);
		while (f.exists()) {
			count++;
			tempFilename = getTempFilename(count);
			f = new File(tempFilename);
		}
		return getTempFilename(count);
	}

	private static String getTempFilename(int c) {
		return System.getProperty("java.io.tmpdir") + System.getProperty("file.separator") + System.currentTimeMillis()
				+ "X" + c + "." + FC_TEMPFILE_EXT;
	}

	public static String xmlToString(Document xml) {
		XMLOutputter xmlOut = new XMLOutputter();
		xmlOut.setFormat(Format.getPrettyFormat());
		return xmlOut.outputString(xml);
	}

	public static String xmlToString(Element elem) {
		return XMLTools.xmlToString(elem, true);
	}

	/**
	 *
	 * @param elem
	 * @param showXmlHeader
	 *            show the xml-version header or not
	 * @return
	 */
	public static String xmlToString(Element elem, boolean showXmlHeader) {
		XMLOutputter xmlOut = new XMLOutputter();
		xmlOut.setFormat(Format.getPrettyFormat());
		if (showXmlHeader) {
			return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + xmlOut.outputString(elem);
		} else {
			return xmlOut.outputString(elem);
		}
	}

	/**
	 *
	 * @param bool
	 * @return Yes or No wheter bool is true or false.
	 */
	public static String boolToYesNo(boolean bool) {
		if (bool) {
			return "yes";
		} else {
			return "no";
		}
	}

	/**
	 * Just return n of the given char as a String. Used for indenting.
	 * @param c
	 * @param n
	 * @return
	 */
	public static String getNumChars(char c, int n) {
		String str = "";
		for (int i = 0; i < n; i++) {
			str += c;
		}
		return str;
	}

}
